@@ -1,6 +1,5 @@
 from mock import patch
 import time
-from optparse import OptionParser
 
 try:
     import simplejson as json
@@ -18,8 +17,9 @@ except ImportError:
 from test.discovery_failure_test import BrokenImportTestCase
 from test.test_logger_test import ExceptionInClassFixtureSampleTests
 from test.test_case_test import RegexMatcher
-from testify import TestCase, assert_equal, assert_gt, assert_in,  assert_in_range, setup_teardown
+from testify import TestCase, assert_equal, assert_gt, assert_in,  assert_in_range, assert_raises, setup_teardown
 from testify.plugins.sql_reporter import add_command_line_options, SQLReporter
+from testify.test_program import default_parser
 from testify.test_result import TestResult
 from testify.test_runner import TestRunner
 
@@ -47,7 +47,7 @@ class SQLReporterBaseTestCase(TestCase):
             msg = 'SQL Reporter plugin requires sqlalchemy and you do not have it installed in your PYTHONPATH.\n'
             raise ImportError, msg
 
-        parser = OptionParser()
+        parser = default_parser()
         add_command_line_options(parser)
         self.fake_buildbot_run_id = 'A' * 36
         (options, args) = parser.parse_args([
@@ -227,6 +227,50 @@ class SQLReporterTestCase(SQLReporterBaseTestCase):
         assert_in('Exception truncated.', failure.error)
 
 
+class RetryTestCase(SQLReporterBaseTestCase):
+    @setup_teardown
+    def patch_engine(self):
+        class BadConnector(object):
+            def __init__(self, connect, max_failures=10):
+                self.connect = connect
+                self.failures = 0
+                self.max_failures = 10
+
+            def __call__(self, *args, **kwargs):
+                if self.failures < self.max_failures:
+                    self.failures += 1
+                    # needs three arguments for i-dont-care reason.
+                    raise SA.exc.OperationalError(1, 2, 3)
+                else:
+                    return self.connect(*args, **kwargs)
+
+        self.reporter.retry_interval = .001
+        self.reporter.retry_backoff = .001
+        with patch.object(
+            self.reporter.engine,
+            'connect',
+            BadConnector(self.reporter.engine.connect),
+        ) as self.bad_connector:
+            yield
+
+    def test_can_succeed(self):
+        self.reporter.retry_limit = .046
+
+        assert self.bad_connector.failures == 0
+        conn = self.reporter._connect()
+        assert self.bad_connector.failures == 10
+
+        assert conn
+        assert type(conn) is SA.engine.base.Connection
+
+    def test_can_fail(self):
+        self.reporter.retry_limit = .045
+
+        assert self.bad_connector.failures == 0
+        with assert_raises(SA.exc.OperationalError):
+            self.reporter._connect()
+
+
 class SQLReporterDiscoveryFailureTestCase(SQLReporterBaseTestCase, BrokenImportTestCase):
     def test_sql_reporter_sets_discovery_failure_flag(self):
         runner = TestRunner(self.broken_import_module, test_reporters=[self.reporter])
