@@ -25,7 +25,8 @@ except ImportError:
 import yaml
 import time
 import threading
-import Queue
+import Queue as queue
+from Queue import Queue
 
 SA = None
 try:
@@ -39,6 +40,21 @@ def md5(s):
     return hashlib.md5(s.encode('utf8') if isinstance(s, unicode) else s).hexdigest()
 
 
+class TaskQueue(Queue):
+    def __init__(self, maxsize=0, worker_function=lambda: None):
+        self._func = worker_function
+        self._thread = None
+        # Queue is apparently an old-style class.
+        Queue.__init__(self, maxsize)
+
+    def put(self, item, block=True, timeout=None):
+        if self._thread is None:
+            self._thread = threading.Thread(target=self._func)
+            self._thread.daemon = True
+            self._thread.start()
+        Queue.put(self, item, block, timeout)
+
+
 class SQLReporter(test_reporter.TestReporter):
 
     def __init__(self, options, *args, **kwargs):
@@ -49,9 +65,13 @@ class SQLReporter(test_reporter.TestReporter):
             'pool_recycle' : 3600,
         })
 
+        self.retry_limit = options.retry_limit
+        self.retry_interval = options.retry_interval
+        self.retry_backoff = options.retry_backoff
+
         self.init_database()
         self.engine = SA.create_engine(dburl, **create_engine_opts)
-        self.conn = self.engine.connect()
+        self.conn = self._connect()
         self.metadata.create_all(self.engine)
 
         if not options.build_info:
@@ -67,18 +87,32 @@ class SQLReporter(test_reporter.TestReporter):
                 for row in self.conn.execute(self.Tests.select())
             )
 
-        self.result_queue = Queue.Queue()
+        self.result_queue = TaskQueue(worker_function=self.report_results)
         self.ok = True
 
         self.reporting_frequency = options.sql_reporting_frequency
         self.batch_size = options.sql_batch_size
-
-        self.reporting_thread = threading.Thread(target=self.report_results)
-        self.reporting_thread.daemon = True
-        self.reporting_thread.start()
-
         super(SQLReporter, self).__init__(options, *args, **kwargs)
 
+    def _connect(self):
+        """Return an SA connection, based on self.engine.
+        The connection will be retried a limited number of times if necessary.
+        """
+        wait = self.retry_interval
+        limit = self.retry_limit
+
+        while True:
+            try:
+                return self.engine.connect()
+            except SA.exc.OperationalError:
+                if limit <= 0:
+                    raise
+                else:
+                    logging.warning('SQL connection failed, retrying in %g seconds (giving up in %g seconds)...', wait, limit)
+                    time.sleep(min(wait, limit))
+                    limit -= wait
+                    wait += self.retry_backoff
+
     def init_database(self):
         self.metadata = SA.MetaData()
 
@@ -271,7 +305,7 @@ class SQLReporter(test_reporter.TestReporter):
     def report_results(self):
         """A worker func that runs in another thread and reports results to the database.
         Create a self.TestResults row from a test result dict. Also inserts the previous_run row."""
-        conn = self.engine.connect()
+        conn = self._connect()
 
         while True:
             results = []
@@ -282,7 +316,7 @@ class SQLReporter(test_reporter.TestReporter):
             try:
                 while True:
                     results.append(self.result_queue.get_nowait())
-            except Queue.Empty:
+            except queue.Empty:
                 pass
 
             # Insert any previous runs, if necessary.
